package PayServer

import (
	"crypto/sha1"
	"encoding/hex"
	"encoding/json"
	"fmt"
	"game/base/log"
	"game/base/redis"
	"game/base/util"
	"game/config"
	"game/excel"
	"io/ioutil"
	"net/http"
	"net/url"
	"strings"
	"time"
)

type HGamePreOrderRet struct {
	GameKey     string `json:"game_key"`
	OpenId      string `json:"open_id"`
	TotalFee    string `json:"total_fee"`
	GameOrderno string `json:"game_orderno"`
	Subject     string `json:"subject"`
	NotifyUrl   string `json:"notify_url"`
	Timestamp   string `json:"timestamp"`
	Nonce       string `json:"nonce"`
	Signature   string `json:"signature"`
}

func HGamePreOrder(w http.ResponseWriter, r *http.Request) {
	if r.Method != "POST" {
		return
	}
	ret := &HGamePreOrderRet{}
	defer func() {
		data, _ := json.Marshal(ret)
		w.Write(data)
	}()

	r.ParseForm()
	account := r.PostFormValue("account")
	produce := r.PostFormValue("produce")
	uid := r.PostFormValue("uid")
	sid := r.PostFormValue("server")

	if !redis.HExists("game_info", sid) {
		log.Error("not Found server ", r.Form.Encode())
		return
	}
	info := excel.GetPlatformIdById(util.StringToUint32(produce))
	ret.OpenId = account
	ret.GameKey = config.HGameKey
	billNo, _ := util.NextId()
	ret.GameOrderno = fmt.Sprintf("%d|%s|%s|%s", billNo, sid, produce, uid)
	ret.Timestamp = util.Int64ToString(time.Now().Unix())
	ret.Subject = info.Name
	ret.TotalFee = fmt.Sprintf("%.2f", float32(info.Money))
	if config.CmConfig.IsHGameDebug {
		ret.TotalFee = fmt.Sprintf("%.2f", 0.01)
	}
	ret.Nonce = util.GenRandomString()
	ret.NotifyUrl = config.CmConfig.AppPayUrl + "/hGameCallBack"
	values := url.Values{}
	values.Add("game_key", ret.GameKey)
	values.Add("open_id", ret.OpenId)
	values.Add("total_fee", ret.TotalFee)
	values.Add("game_orderno", ret.GameOrderno)
	values.Add("subject", ret.Subject)
	values.Add("notify_url", ret.NotifyUrl)
	values.Add("timestamp", ret.Timestamp)
	values.Add("nonce", ret.Nonce)
	ret.Signature = HGameSign(values, config.HGameSecret)
	log.Info("HGamePreOrder ret:", ret)
}
func HGameSign(values url.Values, secret string) string {
	t1, _ := url.QueryUnescape(values.Encode())
	signStr := t1 + secret
	h := sha1.New()
	h.Write([]byte(signStr))
	ss := h.Sum(nil)
	return hex.EncodeToString(ss)
}

type HGameCallbackRet struct {
	Errcode int    `json:"code"`
	ErrMsg  string `json:"message"`
}

func (srv *PayServer) HGamePayCallBack(w http.ResponseWriter, r *http.Request) {
	ret := &HGameCallbackRet{}
	defer func() {
		data, _ := json.Marshal(ret)
		w.Write(data)
	}()

	r.ParseForm()
	sign := r.Form.Get("signature")
	r.Form.Del("signature")
	if sign != HGameSign(r.Form, config.HGameSecret) {
		ret.Errcode = 100
		ret.ErrMsg = "簽名錯誤"
		return
	}
	cpUserInfo := strings.Split(r.FormValue("game_orderno"), "|")
	sid := util.StringToInt(cpUserInfo[1])
	produce := util.StringToUint32(cpUserInfo[2])
	info := excel.GetPlatformIdById(produce)
	msg := &RechargeCommonC2SMsg{
		RoleId:     util.StringToUint64(cpUserInfo[3]),
		Rmb:        float32(info.Money),
		ChargeYb:   info.Money * 10,
		GiftYb:     0,
		OrderId:    cpUserInfo[0],
		POrderId:   r.FormValue("orderno"),
		PlatformIp: r.Header.Get("X-Real_IP"),
		AttType:    info.Type,
		AttValue:   info.Index,
	}
	ret.Errcode, ret.ErrMsg = srv.DeliverGood(sid, msg)
	log.Infof("HGameCallback DeliverRecv sid:%d Role:%s cpOrder:%s order:%s Rmb:%d product:%d ret %d",
		sid, cpUserInfo[3], msg.OrderId, msg.POrderId, info.Money, produce, ret.Errcode)
	if ret.Errcode == 1 || ret.Errcode == 1002 {
		ret.Errcode = 0
	}
	return
}

type HGameLoginRet struct {
	Code    int             `json:"code"`
	Message string          `json:"message"`
	Data    *HGameLoginData `json:"data"`
}
type HGameLoginData struct {
	OpenId   string `json:"open_id"`
	Avatar   string `json:"avatar"`
	Nickname string `json:"nickname"`
	Gender   string `json:"gender"`
}

func HGameLogin(w http.ResponseWriter, r *http.Request) {
	r.ParseForm()
	res := &HGameLoginRet{}
	defer func() {
		d, _ := json.Marshal(res)
		w.Write(d)
	}()
	loginType := r.FormValue("login_type")
	ticket := r.FormValue("login_ticket")
	timestamp := r.FormValue("timestamp")
	nonce := r.FormValue("nonce")
	value := url.Values{}
	value.Add("game_key", config.HGameKey)
	value.Add("timestamp", timestamp)
	value.Add("nonce", nonce)
	value.Add("login_type", loginType)
	value.Add("login_ticket", ticket)
	value.Add("signature", HGameSign(value, config.HGameSecret))
	p, _ := url.PathUnescape(value.Encode())
	sUrl := "https://gc.hgame.com/user/getticketuserinfo?" + p
	resp, e := http.Get(sUrl)
	if e != nil {
		res.Code = 100
		res.Message = e.Error()
		return
	}
	body, e := ioutil.ReadAll(resp.Body)
	resp.Body.Close()
	e = json.Unmarshal(body, res)
	if e != nil {
		res.Code = 101
		res.Message = e.Error()
		log.Error(e)
		return
	}
	log.Infof("HGameLogin Res Code %d Message %s data %+v", res.Code, res.Message, res.Data)
	return
}
