package log

import (
	"context"
	"fmt"
	"github.com/olivere/elastic/v7"
	"github.com/sirupsen/logrus"
	"gopkg.in/sohlich/elogrus.v7"
	"strings"
	"time"
)

// elastic hook
func initElasticHook(url, host, logIndex, srvName string) (logrus.Hook, error) {
	client, err := elastic.NewClient(elastic.SetURL(url))
	if err != nil {
		logrus.Error(err)
		return nil, err
	}
	_, _, err = client.Ping(url).Do(context.Background())
	if err != nil {
		logrus.Error(err)
		return nil, err
	}

	hook, err := elogrus.NewAsyncElasticHook(client, host, logrus.InfoLevel, strings.ToLower(logIndex+"-"+srvName))
	if err != nil {
		logrus.Error(err)
		return nil, err
	}
	return hook, nil
}

// elasticPl hook
func initElasticPlHook(url, host, logIndex, srvName string) (logrus.Hook, error) {
	client, err := elastic.NewClient(elastic.SetURL(url))
	if err != nil {
		logrus.Error(err)
		return nil, err
	}
	_, _, err = client.Ping(url).Do(context.Background())
	if err != nil {
		logrus.Error(err)
		return nil, err
	}

	hook, err := NewAsyncElasticHook(client, host, logrus.InfoLevel,
		func() string { return strings.ToLower(logIndex + "-" + srvName) },
		func() string { return pipelineName })
	if err != nil {
		logrus.Error(err)
		return nil, err
	}
	return hook, nil
}

var (
	// ErrCannotCreateIndex Fired if the index is not created
	ErrCannotCreateIndex = fmt.Errorf("cannot create index")
)

// IndexNameFunc get index name
type IndexNameFunc func() string

type fireFunc func(entry *logrus.Entry, hook *ElasticHook) error

// ElasticHook is a logrus
// hook for ElasticSearch
type ElasticHook struct {
	client    *elastic.Client
	host      string
	pipeline  IndexNameFunc
	index     IndexNameFunc
	levels    []logrus.Level
	ctx       context.Context
	ctxCancel context.CancelFunc
	fireFunc  fireFunc
}

type message struct {
	Host      string
	Timestamp string `json:"@timestamp"`
	Message   string
	Data      logrus.Fields
	Level     string
}

func NewAsyncElasticHook(client *elastic.Client, host string, level logrus.Level, indexFunc, pipelineFunc IndexNameFunc) (*ElasticHook, error) {
	var levels []logrus.Level
	for _, l := range logrus.AllLevels {
		if l <= level {
			levels = append(levels, l)
		}
	}

	ctx, cancel := context.WithCancel(context.TODO())

	// Use the IndexExists service to check if a specified index exists.
	exists, err := client.IndexExists(indexFunc()).Do(ctx)
	if err != nil {
		// Handle error
		cancel()
		return nil, err
	}
	if !exists {
		createIndex, err := client.CreateIndex(indexFunc()).Do(ctx)
		if err != nil {
			cancel()
			return nil, err
		}
		if !createIndex.Acknowledged {
			cancel()
			return nil, ErrCannotCreateIndex
		}
	}

	return &ElasticHook{
		client:    client,
		host:      host,
		pipeline:  pipelineFunc,
		index:     indexFunc,
		levels:    levels,
		ctx:       ctx,
		ctxCancel: cancel,
		fireFunc:  syncFireFunc,
	}, nil
}

func syncFireFunc(entry *logrus.Entry, hook *ElasticHook) error {
	go func(entry *logrus.Entry, hook *ElasticHook) {
		_, err := hook.client.
			Index().
			Index(hook.index()).
			Type("log").
			Pipeline(hook.pipeline()).
			BodyJson(*createMessage(entry, hook)).
			Do(hook.ctx)

		if err != nil {
			logrus.Error("syncFireFunc err:", err)
			return
		}
	}(entry, hook)

	return nil
}

func createMessage(entry *logrus.Entry, hook *ElasticHook) *message {
	level := entry.Level.String()

	if e, ok := entry.Data[logrus.ErrorKey]; ok && e != nil {
		if err, ok := e.(error); ok {
			entry.Data[logrus.ErrorKey] = err.Error()
		}
	}

	return &message{
		hook.host,
		entry.Time.UTC().Format(time.RFC3339Nano),
		entry.Message,
		entry.Data,
		strings.ToUpper(level),
	}
}

// Fire is required to implement
// Logrus hook
func (hook *ElasticHook) Fire(entry *logrus.Entry) error {
	return hook.fireFunc(entry, hook)
}

// Levels Required for logrus hook implementation
func (hook *ElasticHook) Levels() []logrus.Level {
	return hook.levels
}

// Cancel all calls to elastic
func (hook *ElasticHook) Cancel() {
	hook.ctxCancel()
}
